using System;
using System.IO;
using System.Collections;
using System.Text;
using System.Xml;

namespace RSSReader.OPML
{

	/// <summary>
	/// Contains functionality to read and write OPML files, used
	/// for storage of RSS subscription information.
	/// </summary>
	public class OPMLFile
	{

		#region Private member variables 

		public OPMLNodeCollection nodes;

		#endregion 

		#region Constructor 

		private OPMLFile() {}

		public OPMLFile( string Filename )
		{
			ReadFromFile( Filename );
		}

		#endregion 

		#region Public properties 

		public OPMLNodeCollection Nodes
		{
			get { return nodes; }
		}

		#endregion 

		#region Public methods 
 
		public bool PathExists( string Path )
		{
			return PathExists( nodes, Path );
		}

		public bool IsSubscribed( string FeedUrl )
		{
			return IsSubscribed( nodes, FeedUrl );
		}

		public void ReadFromFile( string Filename )
		{

			nodes = new OPMLNodeCollection();
			nodes.InsertComplete += new RSSReader.OPML.OPMLNodeCollection.OPMLNodeCollectionChangedEventHandler(nodes_InsertComplete);

			if( !File.Exists( Filename ) )
				return;

			using( StreamReader reader = File.OpenText( Filename ) )
			{

				string xmlData = reader.ReadToEnd();

				XmlDocument doc = new XmlDocument();
				doc.LoadXml( xmlData );

				if( doc.DocumentElement.Name != "opml" )
					throw new Exception( "Invalid OPML file: Invalid root element <" + doc.DocumentElement.Name + ">" );

				XmlNode body = doc.SelectSingleNode( "//body" );
				if( body == null )
					throw new Exception( "Invalid OPML file: <body> element not found" );

				foreach( XmlNode xmlNode in body.ChildNodes )
					if( xmlNode.Name == "outline" )
					{
						nodes.Add( OPMLNode.FromXml( xmlNode ) );
					}

			}

			nodes.Sort();

		}

		public void SaveToFile( string Filename )
		{

			nodes.Sort();

			StringBuilder buffer = new StringBuilder();
			XmlTextWriter writer = XmlHelper.CreateXmlWriter( new StringWriter( buffer ) );

			writer.WriteStartElement( "opml" );
			writer.WriteAttributeString("xmlns", "xsd", null, "http://www.w3.org/2001/XMLSchema" );
			writer.WriteAttributeString("xmlns", "xsi", null, "http://www.w3.org/2001/XMLSchema-instance" );

			writer.WriteStartElement( "body" );

			foreach( OPMLNode node in nodes )
				node.WriteToXml( writer );

			writer.WriteEndElement();

			writer.WriteEndElement();

			if( File.Exists( Filename ) )
				File.Delete( Filename );

			using( StreamWriter output = File.CreateText( Filename ) )
			{
				output.WriteLine( buffer.ToString() );
			}

		}

		public ArrayList GetAllSubscriptions()
		{

			ArrayList results = new ArrayList();
			GetAllSubscriptions( results, this.Nodes );
			
			return results;

		}

		private void GetAllSubscriptions( ArrayList results, OPMLNodeCollection nodes )
		{

			foreach( OPMLNode node in nodes )
			{

				if( node is OPMLItemNode )
					results.Add( node );
				else
					GetAllSubscriptions( results, ((OPMLCategoryNode) node).Nodes );

			}

		}

		#endregion 

		#region Private utility methods 

		private bool PathExists( OPMLNodeCollection collection, string Path )
		{

			foreach( OPMLNode node in collection )
			{

				if( node.Path.ToLower() == Path )
					return true;

				if( node is OPMLCategoryNode )
				{

					bool pathExists = PathExists( ((OPMLCategoryNode) node).Nodes, Path );
					if( pathExists )
						return true;

				}

			}

			return false;

		}

		private bool IsSubscribed( OPMLNodeCollection collection, string feedUrl )
		{

			foreach( OPMLNode node in collection )
			{

				if( node is OPMLItemNode )
				{

					if( ((OPMLItemNode) node).XmlUrl.ToLower() == feedUrl.ToLower() )
						return true;

				}
				else
				{

					bool isSubscribed = IsSubscribed( ((OPMLCategoryNode) node).Nodes, feedUrl );
					if( isSubscribed )
						return true;

				}

			}

			return false;

		}

		#endregion

		#region Nodes collection event handlers 

		private void nodes_InsertComplete(object sender, OPMLNode value)
		{
			value.Collection = this.nodes;
			value.Parent = null;
		}

		#endregion 

	}

}
