using System;
using System.Collections;
    
namespace RSSReader.RSS
{

    /// <summary>
    /// Represents a collection of <see cref='RSSReader.RSS.RSSItem'/> objects.
    /// </summary>
    public class RSSItemCollection : System.Collections.ICollection
    {

		#region Private variables 
        
        // Provides the base object storage used by this class
        private System.Collections.ArrayList items;
        
        // This version counter is incremented any time the collection changes.
        private int listVersion;

		#endregion 

		#region Constructor 
        
        /// <summary>
        /// Initializes a new instance of the <see cref='RSSReader.RSS.RSSItemCollection'/> class.
        /// </summary>
        internal RSSItemCollection()
        {
            items = new System.Collections.ArrayList();
            listVersion = 0;
        }

		/// <summary>
		/// Internal method called whenever the collection changes to increment the Version property.
		/// </summary>
		private void IncrementVersion()
		{
			listVersion = (listVersion + 1);
		}
        
		#endregion 

		#region Internal properties 
        
        /// <summary>
        /// Returns a number that can be used to determine if the collection has changed since some previous
        /// reference point, since this value changes each time the collection is modified in any way.
        /// </summary>
        internal int Version
        {
            get
            {
                return listVersion;
            }
        }

		#endregion 

		#region Collection members 
        
        /// <summary>
        /// Gets the number objects contained in the collection.
        /// <summary>
        public virtual int Count
        {
            get
            {
                return items.Count;
            }
        }
        
        /// <summary>
        /// Gets a value indicating whether access to the collection is synchronized (thread-safe).
        /// <summary>
        public virtual bool IsSynchronized
        {
            get
            {
                return items.IsSynchronized;
            }
        }
        
        /// <summary>
        /// Gets an object that can be used to synchronize access to the collection.
        /// <summary>
        public virtual object SyncRoot
        {
            get
            {
                return items.SyncRoot;
            }
        }
        
        /// <summary>
        /// Returns the <see cref='RSSReader.RSS.RSSItem'/> at the specified index of the <see cref='RSSReader.RSS.RSSItemCollection'/>.
        /// </summary>
        /// <param name='index'>The zero-based index of the <see cref='RSSReader.RSS.RSSItem'/> to locate in the collection.</param>
        /// <returns> The <see cref='RSSReader.RSS.RSSItem'/> at the specified index of the collection.</returns>
        /// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
        public virtual RSSItem this[int index]
        {
            get
            {
                return ((RSSItem)(this.items[index]));
            }
        }
        
        /// <summary>
        /// Returns a <see cref='System.Collections.IEnumerator'/> object that can be used to enumerate
        /// the entire collection.
        /// </summary>
        public virtual System.Collections.IEnumerator GetEnumerator()
        {
            return items.GetEnumerator();
        }
        
        /// <summary>
        /// Copies the elements of this RSSReader.RSS.RSSItemCollection to a System.Array, starting at a particular System.Array index.
        /// <summary>
        public virtual void CopyTo(System.Array array, int index)
        {
            items.CopyTo(array, index);
        }
        
        /// <summary>
        /// Adds the specified <see cref='RSSReader.RSS.RSSItem'/> to the <see cref='RSSReader.RSS.RSSItemCollection'/> collection.
        /// </summary>
        /// <returns>The position into which the new element was inserted.</returns>
        public virtual int Add(RSSItem value)
        {
            // Inserting
            this.IncrementVersion();
            int retVal = items.Add(value);
            // InsertComplete
            return retVal;
        }
        
        /// <summary>
        /// Removes the specified <see cref='RSSReader.RSS.RSSItem'/> from the <see cref='RSSReader.RSS.RSSItemCollection'/> collection.
        /// </summary>
        public virtual void Remove(RSSItem value)
        {
            // Removing
            this.IncrementVersion();
            items.Remove(value);
            // RemoveComplete
        }
        
        /// <summary>
        /// Removes the <see cref='RSSReader.RSS.RSSItem'/> object at the specified index from the <see cref='RSSReader.RSS.RSSItemCollection'/> collection.
        /// </summary>
        public virtual void RemoveAt(int index)
        {
            object obj = items[index];
            this.Remove(((RSSItem)(obj)));
        }
        
        /// <summary>
        /// Inserts a <see cref='RSSReader.RSS.RSSItem'/> object into the <see cref='RSSReader.RSS.RSSItemCollection'/> collection at the specified position.
        /// </summary>
        /// <param name='index'>The zero-based index at which value should be inserted.</param>
        /// <param name='value'>The RSSReader.RSS.RSSItem to insert into the <see cref='RSSReader.RSS.RSSItemCollection'/> collection.</param>
        public virtual void Insert(int index, RSSItem value)
        {
            // Inserting
            this.IncrementVersion();
            items.Insert(index, value);
            // InsertComplete
        }
        
        /// <summary>
        /// determines whether the <see cref='RSSReader.RSS.RSSItemCollection'/> collection contains a specific <see cref='RSSReader.RSS.RSSItem'/> object.
        /// </summary>
        /// <param name='value'>The RSSReader.RSS.RSSItem object to locate in the <see cref='RSSReader.RSS.RSSItemCollection'/> collection.</param>
        /// <returns>True if the System.Object is found in the collection; otherwise, False.</returns>
        public virtual bool Contains(RSSItem value)
        {
            return items.Contains(value);
        }
        
        /// <summary>
        /// Removes all items from the collection.
        /// </summary>
        public virtual void Clear()
        {
            // Clearing
            this.IncrementVersion();
            items.Clear();
            // ClearComplete
        }
        
        /// <summary>
        /// Creates a shallow copy of the collection.
        /// </summary>
        public virtual RSSItemCollection Clone()
        {
            RSSItemCollection obj = new RSSItemCollection();
            obj.items = ((System.Collections.ArrayList)(items.Clone()));
            obj.listVersion = this.listVersion;
            return obj;
        }

		#endregion 

		#region Custom sort routines 

		internal void Sort( ItemSortField SortField, ItemSortOrder SortOrder )
		{

			foreach( RSSItem item in items )
			{
				item.SortField = SortField;
				item.SortOrder = SortOrder;
			}

			items.Sort();

			foreach( RSSItem item in items )
			{

				if( item.Comments != null )
					item.Comments.Sort( SortField, SortOrder );

			}

		}

		#endregion 

    }

}

