#region Copyright Morrison Shwartz, Inc - Author: Jonah Stagner 
/////////////////////////////////////////////////////////////////////////
// Copyright Morrison Schwartz, Inc 2003-2004, All rights reserved     //
//---------------------------------------------------------------------//
//Author: Jonah L. Stagner                                             //
//---------------------------------------------------------------------//
// This product is protected by US Copyright laws and international    //
// treaties.  For more information, see the End User License Agreement //
// in the application installation folder.                             //
//---------------------------------------------------------------------//
// Visit us on the web at: http://www.ms-inc.net                       //
/////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.IO;
using System.Xml;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.Diagnostics;

using SharpTools;
using SharpTools.IDE;
using SharpTools.DOM;
using SharpTools.PluginInterface;

namespace ScreenCapture
{

	/// <summary>
	/// Allows the developer to create screen captures of the application currently 
	/// running.
	/// </summary>
	public class ScreenCapturePlugin : ISharpToolsPlugin
	{

		#region Private data members 

		private SharpTools.Toolbar			toolbar;
		private SharpTools.ToolbarButton	cmdRun;
		private SharpTools.ToolbarButton	cmdShowCaptures;
		private SharpTools.ToolbarCombo		captureType;
		private SharpTools.IDE.Toolwindow	toolwindow;
		private Captures					captures;
		private HotKeyForm					hotkey;

		#endregion 

		#region Constants 

		private const string CT_DESKTOP = "Entire Screen";
		private const string CT_ACTIVE = "Active Window";
		private const string CT_SELECT = "Select the window object with mouse";
		private const string TOOLWINDOW_GUID = "{4A3840E0-0C5D-43f8-B03C-025B32BE72B4}";

		#endregion 

		#region Constructor 

		public ScreenCapturePlugin()
		{
		}

		#endregion 

		#region ISharpToolsPlugin Members 

		public string Description
		{
			get 
			{
				return "Screen capture utility.  This plugin can capture snapshots of the entire screen, the " + 
					"active window, a window object selected with the mouse, or the program that is currently " + 
					"running from the VS.NET IDE.  This plugin supports the CTRL-SHIFT-C hotkey, allowing it " +
					"to capture screenshots with the press of a key combination, even when the VS.NET IDE is not" +
					"the active window."; 
			}
		}

		public string ID
		{
			get { return "SharpTools.Plugins.ScreenCapture"; }
		}

		public string Name
		{
			get { return "SharpTools Screen Capture"; }
		}

		public void OnConnect()
		{
			
			// Attach this plugin to the SharpTools toolbar
			toolbar = new SharpTools.Toolbar( "SharpTools.Plugins.ScreenCapture" );

			// Create our Run button
			cmdRun = (SharpTools.ToolbarButton) toolbar.Controls.Add( ControlType.Button, "ScreenCapture" );
			cmdRun.Caption = "Capture";
			cmdRun.Tooltip = "Create screen capture (CTRL-SHIFT-C)";
			cmdRun.ButtonStyle = ButtonStyle.IconAndCaption;
			cmdRun.Picture = 280;
			cmdRun.Click += new EventHandler(cmdRun_Click);

			// Create the "Capture Type" dropdown
			captureType = (SharpTools.ToolbarCombo) toolbar.Controls.Add( ControlType.Dropdown, "captureType" );
			captureType.Caption = "Capture Type";
			captureType.ShowLabel = false;
			captureType.Tooltip = "Select the screen capture method";
			captureType.BeginGroup = false;
			captureType.DropDownWidth = 200;
			LoadCaptureTypes();

			// Create the "Show Captures" button.  The problem with ToolWindow controls is that VS.NET tries to
			// make them context-sensitive in the sense that if you activate one during debug mode, it auto-hides
			// when you return to design mode.  Interesting, but unwanted in this situation.
			cmdShowCaptures = (SharpTools.ToolbarButton) toolbar.Controls.Add( ControlType.Button, "ShowCaptures" );
			cmdShowCaptures.Caption = "Show Captures";
			cmdShowCaptures.Tooltip = "Show Captured Images";
			cmdShowCaptures.Picture = 178;	

			cmdShowCaptures.Click += new EventHandler(cmdShowCaptures_Click);

			// Create the Toolwindow that holds captured images
			captures = new Captures();
			toolwindow = new Toolwindow( new System.Guid( TOOLWINDOW_GUID ), "Screen Captures", captures );

			// Create a HotKeyForm to receive hotkey notifications
			hotkey = new HotKeyForm();
			hotkey.HotKeyPressed += new ScreenCapture.HotKeyForm.HotKeyPressedEventHandler(hotkey_HotKeyPressed);

			Solution.GetInstance().Opened += new EventHandler(ScreenCapturePlugin_Opened);
			Solution.GetInstance().BeforeClosing += new EventHandler(ScreenCapturePlugin_BeforeClosing);
			Solution.GetInstance().ProjectAdded += new ProjectAddedEventHandler(ScreenCapturePlugin_ProjectAdded);
			Solution.GetInstance().ProjectRemoved += new ProjectRemovedEventHandler(ScreenCapturePlugin_ProjectRemoved);
			Solution.GetInstance().ProjectRenamed += new ProjectRenamedEventHandler(ScreenCapturePlugin_ProjectRenamed);

		}

		public void OnDisconnect(SharpTools.PluginInterface.DisconnectMode disconnectMode)
		{
			
			try
			{
				hotkey.Close();
				hotkey = null;
			}
			catch( Exception ) { }

		}

		#endregion

		private void cmdRun_Click(object sender, EventArgs e)
		{
			CaptureSnapshot();
		}

		private void hotkey_HotKeyPressed(object sender, ScreenCapture.HotKeyForm.HotKeyPressedEventArgs e)
		{
			CaptureSnapshot();
		}

		private IntPtr GetApplicationWindow()
		{

			string appName = "your program";

			if( Globals.ActiveProject == null )
			{
				MessageBox.Show( "This plugin captures screenshots of your program running in debug mode.  Please load a project and run it before using this plugin.", "SharpTools Screen Capture" );
				return IntPtr.Zero;
			}

			appName = Path.GetFileName( Solution.GetInstance().Projects[ Solution.GetInstance().StartupProject ].AssemblyName );

			SharpTools.IDE.Debugger debugger = SharpTools.IDE.Debugger.GetInstance();
			if( debugger.CurrentMode != DebuggerMode.Run )
			{
				MessageBox.Show( "You are currently in " + debugger.CurrentMode.ToString().ToLower() + " mode.  Please enter run mode and use this plugin to capture snapshots of " + appName + ".", "SharpTools Screen Capture" );
				return IntPtr.Zero;
			}

			EnvDTE.Debugger debugObj = (EnvDTE.Debugger) debugger.Object;
			EnvDTE.Processes processList = debugObj.DebuggedProcesses;
			EnvDTE.Process currentProcess = processList.Item( 1 );
			int processID = currentProcess.ProcessID;
			
			System.Diagnostics.Process process = System.Diagnostics.Process.GetProcessById( processID );
			IntPtr hwnd = process.MainWindowHandle;
			if( hwnd == IntPtr.Zero )
			{
				MessageBox.Show( appName + " does not have a main window, or the main window handle could not be retrieved.", "SharpTools Screen Capture" );
				return IntPtr.Zero;
			}

			Capture.ActivateWindow( hwnd );
			return hwnd;

		}

		private void CaptureSnapshot()
		{

			try
			{

				IntPtr hwnd = IntPtr.Zero;
				
				switch( captureType.ListIndex )
				{

					case 1:
						hwnd = Capture.GetDesktopWindow();
						break;

					case 2:
						hwnd = Capture.GetForegroundWindow();
						break;

					case 3:
						StartSelectCapture.SelectWindow( new StartSelectCapture.WindowSelectedHandler( CaptureFromWindow ) );
						return;

					default:
						hwnd = GetApplicationWindow();
						break;

				}

				if( hwnd != IntPtr.Zero )
					CaptureFromWindow( hwnd );

			}
			catch( Exception err )
			{
				MessageBox.Show( "There was an error capturing a snapshot: " + err.Message, "SharpTools Screen Capture" );
			}

		}

		private void CaptureFromWindow( IntPtr hwnd )
		{

			if( hwnd == IntPtr.Zero )
				return;

			Image img = Capture.CaptureWindow( hwnd );
			captures.AddCapture( img );

			if( !toolwindow.Visible )
				toolwindow.Activate();

		}

		private void LoadCaptureTypes( bool SetStartupProjectName )
		{

			captureType.Clear();

			try
			{

				string appName = "Startup Project";

				if( SetStartupProjectName && Globals.ActiveProject != null )
				{
				
					string startup = Solution.GetInstance().StartupProject ;
					if( startup != "" )
					{
						string assemblyName = Solution.GetInstance().Projects[ startup ].AssemblyName;
						appName = Path.GetFileName( assemblyName );
					}

				}

				captureType.AddItem( appName );

			}
			catch( Exception )
			{
			}

			captureType.AddItem( CT_DESKTOP );
			captureType.AddItem( CT_ACTIVE );
			captureType.AddItem( CT_SELECT );

		}

		private void LoadCaptureTypes()
		{
			LoadCaptureTypes( true );
		}

		#region Solution event handlers 

		private void ScreenCapturePlugin_Opened(object sender, EventArgs e)
		{
			LoadCaptureTypes();
		}

		private void ScreenCapturePlugin_BeforeClosing(object sender, EventArgs e)
		{
			LoadCaptureTypes( false );
		}

		private void ScreenCapturePlugin_ProjectAdded(Solution sender, Project project)
		{
			LoadCaptureTypes();
		}

		private void ScreenCapturePlugin_ProjectRemoved(Solution sender, Project project)
		{
			LoadCaptureTypes();
		}

		private void ScreenCapturePlugin_ProjectRenamed(Solution sender, Project project, string oldName)
		{
			LoadCaptureTypes();
		}

		#endregion

		private void cmdShowCaptures_Click(object sender, EventArgs e)
		{

			if( toolwindow != null )
				toolwindow.Activate();

		}
	}

}
